
#ifndef __HSCAL_BLACS_CBLACSGRIDMAP_HH
#define __HSCAL_BLACS_CBLACSGRIDMAP_HH

#include <stdio.h>

#include <stdlib.h>

#include "../../../include/scalapack.h"

void Cblacs_gridmap_nocomm(
           HSCALBLACSCONTEXT **ConTxt, int *usermap, 
           int ldup, int nprow0, int npcol0
)
{
   int i, j, *iptr;
   int nprow, npcol, Ng;
   HSCALBLACSCONTEXT *ctxt;

   nprow = Mpval(nprow0);
   npcol = Mpval(npcol0);
   Ng = nprow * npcol;
   if ( (nprow < 1) || (npcol < 1) )
   {
      BI_BlacsErr_nocomm(
        NULL, -1, "BLACS_GRIDINIT/BLACS_GRIDMAP",
        "Illegal grid (%d x %d), #procs=%d", nprow, npcol, Ng);
   }

/*
 * Form MPI communicator for scope = 'all'
 */
   if (Ng > 2) i = Ng;
   else i = 2;
   iptr = (int *) malloc(i*sizeof(int));
   for (j=0; j < npcol; j++)
   {
      for (i=0; i < nprow; i++) iptr[i*npcol+j] = usermap[j*Mpval(ldup)+i];
   }

/*
 * ==================================================
 * Get new context and add it to my array of contexts
 * ==================================================
 */
   ctxt = (HSCALBLACSCONTEXT *) malloc(sizeof(HSCALBLACSCONTEXT));

   if (ctxt == NULL)
   {
      BI_BlacsErr_nocomm(
        NULL, -1, "BLACS_GRIDMAP",
        "Unable to create BLACS context"
      );

      return;
   }

   *ConTxt = ctxt;

   ctxt->rscp.Np = npcol;
   ctxt->rscp.Iam = -1;
   ctxt->rscp.scope = 'r';
   ctxt->cscp.Np = nprow;
   ctxt->cscp.Iam = -1;
   ctxt->cscp.scope = 'c';
   ctxt->pscp.Np = ctxt->ascp.Np = Ng;
   ctxt->pscp.Iam = ctxt->ascp.Iam = -1;
   ctxt->ascp.scope = 'a';
   ctxt->rscp.scope = 'r';
   ctxt->Nr_bs = ctxt->Nr_co = 1;
   ctxt->Nb_bs = ctxt->Nb_co = 2;
   ctxt->TopsRepeat = ctxt->TopsCohrnt = 0;

   free(iptr);
}

#endif
